import {Component, OnInit} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {
  GetSigningRequestResponse,
  GetSigningRequestResponseSigningRequestDocument, PatchOperation, SigningRequestDocumentService,
  SigningRequestService
} from '../../../gen';
import {PDFSource} from 'ng2-pdf-viewer';

@Component({
  selector: 'app-hello',
  standalone: false,
  templateUrl: './signing.component.html',
  styleUrl: './signing.component.css'
})
export class SigningComponent implements OnInit {
  documents: GetSigningRequestResponseSigningRequestDocument[] = [];
  selectedDocument: GetSigningRequestResponseSigningRequestDocument = {
    confirmed: false, id: '', name: ''
  };
  selectedDocumentPdf: PDFSource | undefined = undefined;

  constructor(private route: ActivatedRoute, private signingRequestService: SigningRequestService, private signingRequestDocumentService: SigningRequestDocumentService) {
  }

  ngOnInit(): void {
    const id: string = this.route.snapshot.paramMap.get("id") || "";

    this.signingRequestService.getSigningRequest(id).subscribe({
      next: (response: GetSigningRequestResponse): void => {
        this.documents = response.signingRequestDocuments || [];
      },
      error(err: Error): void {
        console.log(err);
      }
    });
  }

  setActiveDocument(document: GetSigningRequestResponseSigningRequestDocument): void {
    this.selectedDocument = document;
    this.signingRequestDocumentService.getSigningRequestDocumentData(this.selectedDocument.id).subscribe({
      next: (fileData: Blob): void => {
        this.convertBlobToUint8Array(fileData).then((uInt8Array: Uint8Array) => {
          this.selectedDocumentPdf = {data: uInt8Array};
        });
      },
      error: (err: Error): void => {
        console.log(err);
      }
    })
  }

  private convertBlobToUint8Array(blob: Blob): Promise<Uint8Array> {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.onload = () => {
        const arrayBuffer = reader.result as ArrayBuffer;
        const uint8Array = new Uint8Array(arrayBuffer);
        resolve(uint8Array);
      };
      reader.onerror = () => {
        reject(reader.error);
      };
      reader.readAsArrayBuffer(blob);
    });
  }

  onConfirm(): void {
    this.confirmDocument(this.selectedDocument);
  }

  private confirmDocument(signingRequestDocument: GetSigningRequestResponseSigningRequestDocument): void {
    const patch: PatchOperation = {
      op: "replace", path: "/confirmed", value: "true"
    }
    this.signingRequestDocumentService.patchSigningRequestDocument(signingRequestDocument.id, Array.of(patch)).subscribe({
      next: (): void => {
        signingRequestDocument.confirmed = true;
      },
      error: (err: Error): void => {
        console.log(err);
      }
    });
  }
}
